@@ -1,11 +1,16 @@
+#' Begin a Search for the Optimal Solution
+#' 
 #' This method creates an object of type optimal_experimental_design and will immediately initiate
-#' a search through $1_{T}$ space.
+#' a search through $1_{T}$ space. Since this search takes exponential time, for most machines, 
+#' this method is futile beyond 28 samples. You've been warned!
 #' 
 #' @param X					The design matrix with $n$ rows (one for each subject) and $p$ columns 
 #' 							(one for each measurement on the subject). This is the design matrix you wish 
 #' 							to search for a more optimal design.
-#' @param objective			The objective function to use when greedily searching design space. This is a string
-#' 							"\code{abs_sum_diff}" (default) or "\code{mahal_dist}."
+#' @param objective			The objective function to use when searching design space. This is a string
+#' 							with valid values "\code{mahal_dist}" (the default), "\code{abs_sum_diff}" or "\code{kernel}".
+#' @param Kgram				If the \code{objective = kernel}, this argument is required to be an \code{n x n} matrix whose
+#' 							entries are the evaluation of the kernel function between subject i and subject j. Default is \code{NULL}.
 #' @param wait				Should the \code{R} terminal hang until all \code{max_designs} vectors are found? The 
 #' 							deafult is \code{FALSE}.
 #' @param start				Should we start searching immediately (default is \code{TRUE}).
@@ -14,21 +19,31 @@
 #' 
 #' @author Adam Kapelner
 #' @export
-initOptimalExperimentalDesignObject = function(X,
-		objective = "abs_sum_diff", 
+initOptimalExperimentalDesignObject = function(
+		X = NULL,
+		objective = "mahal_dist", 
+		Kgram = NULL,
 		wait = FALSE, 
 		start = TRUE,
 		num_cores = 1){
-	#get dimensions immediately
-	n = nrow(X)
+	
+	verify_objective_function(objective, Kgram, n)
+	
+	if (!is.null(Kgram)){
+		n = nrow(Kgram)
+		p = NA
+	} else {
+		n = nrow(X)
+		p = ncol(X)
+	}
 	if (n %% 2 != 0){
 		stop("Design matrix must have even rows to have equal treatments and controls")
 	}
-	p = ncol(X)
+	
 	
 	if (objective == "abs_sum_diff"){
 		#standardize it -- much faster here
-		Xstd = apply(X, 2, function(xj){(xj - mean(xj)) / sd(xj)})
+		Xstd = standardize_data_matrix(X)
 	}
 	if (objective == "mahal_dist"){
 		if (p < n){
@@ -45,26 +60,35 @@ initOptimalExperimentalDesignObject = function(X,
 	#now go ahead and create the Java object and set its information
 	java_obj = .jnew("OptimalExperimentalDesign.OptimalExperimentalDesign")
 	.jcall(java_obj, "V", "setNumCores", as.integer(num_cores))
-	.jcall(java_obj, "V", "setNandP", as.integer(n), as.integer(p))
+	.jcall(java_obj, "V", "setN", as.integer(n))
+	if (objective != "kernel"){
+		p = ncol(X)
+		.jcall(java_obj, "V", "setP", as.integer(p))
+	}
 	.jcall(java_obj, "V", "setObjective", objective)
 	if (wait){
 		.jcall(java_obj, "V", "setWait")
 	}	
 	
-	#feed in the data
-	for (i in 1 : n){	
-		if (objective == "abs_sum_diff"){
-			.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), Xstd[i, , drop = FALSE]) #java indexes from 0...n-1
-		} else {
-			.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), X[i, , drop = FALSE]) #java indexes from 0...n-1
+	#feed in the gram matrix if applicable
+	if (!is.null(Kgram)){
+		setGramMatrix(java_obj, Kgram)
+	} else {
+		#feed in the data
+		for (i in 1 : n){	
+			if (objective == "abs_sum_diff"){
+				.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), Xstd[i, , drop = FALSE]) #java indexes from 0...n-1
+			} else {
+				.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), X[i, , drop = FALSE]) #java indexes from 0...n-1
+			}
 		}
-	}
-	
-	#feed in the inverse var-cov matrix
-	if (objective == "mahal_dist"){
-		if (p < n){
-			for (j in 1 : p){
-				.jcall(java_obj, "V", "setInvVarCovRow", as.integer(j - 1), SinvX[j, , drop = FALSE]) #java indexes from 0...n-1
+		
+		#feed in the inverse var-cov matrix
+		if (objective == "mahal_dist"){
+			if (p < n){
+				for (j in 1 : p){
+					.jcall(java_obj, "V", "setInvVarCovRow", as.integer(j - 1), SinvX[j, , drop = FALSE]) #java indexes from 0...n-1
+				}
 			}
 		}
 	}
@@ -121,17 +145,30 @@ summary.optimal_experimental_design_search = function(object, ...){
 
 #' Returns the results (thus far) of the optimal design search
 #' 
-#' @param obj 			The \code{optimal_experimental_design} object that is currently running the search
+#' @param obj 				The \code{optimal_experimental_design} object that is currently running the search
+#' @param num_vectors		How many allocation vectors you wish to return. The default is 1 meaning the best vector. If \code{Inf},
+#' 							it means all vectors.
+#' @param form				Which form should it be in? The default is \code{one_zero} for 1/0's or \code{pos_one_min_one} for +1/-1's.
 #' 
 #' @author Adam Kapelner
 #' @export
-resultsOptimalSearch = function(obj){
-	opt_obj_val = .jcall(obj$java_obj, "D", "getOptObjectiveVal")
-	indicT = .jcall(obj$java_obj, "[I", "getOptIndicT", .jevalArray)
-	all_obj_vals = .jcall(obj$java_obj, "[D", "getAllObjectiveVals", .jevalArray)
+resultsOptimalSearch = function(obj, num_vectors = 1, form = "one_zero"){
+	obj_vals = .jcall(obj$java_obj, "[D", "getAllObjectiveVals", simplify = TRUE)
+	ordered_indices = order(obj_vals)
+	
+	if (num_vectors == Inf){
+		num_vectors = length(ordered_indices)
+	}
+	
+	indicTs = .jcall(obj$java_obj, "[[I", "getIndicTs", as.integer(ordered_indices[1 : num_vectors] - 1), simplify = TRUE)
+	if (form == "pos_one_min_one"){
+		indicTs = (indicTs - 0.5) * 2
+	}
 	list(
-		opt_obj_val = opt_obj_val,
-		indicT = indicT,
-		all_obj_vals = all_obj_vals
+		opt_obj_val = obj_vals[ordered_indices[1]],
+		ordered_obj_vals = obj_vals[ordered_indices],
+		obj_vals_unordered = obj_vals,
+		orig_order = ordered_indices,
+		indicTs = indicTs
 	)
 }
