@@ -1,16 +1,21 @@
+#' Begin a Rerandomization Search
+#' 
 #' This method creates an object of type rerandomization_experimental_design and will immediately initiate
-#' a search through $1_{T}$ space.
+#' a search through $1_{T}$ space for forced-balance designs.
 #' 
 #' @param X							The design matrix with $n$ rows (one for each subject) and $p$ columns 
 #' 									(one for each measurement on the subject). This is the design matrix you wish 
 #' 									to search for a more optimal design.
 #' @param obj_val_cutoff_to_include	Only allocation vectors with objective values lower than this threshold will be returned.
-#' 									The default is \code{NULL} which means all vectors are returned.
+#' 									If the cutoff is infinity, you are doing BCRD and you should use the \code{complete_randomization_with_forced_balanced}
+#' 									function instead.
 #' @param max_designs 				The maximum number of designs to be returned. Default is 10,000. Make this large 
 #' 									so you can search however long you wish as the search can be stopped at any time by
 #' 									using the \code{\link{stopSearch}} method 
-#' @param objective					The objective function to use when searching the design space. This is a string
-#' 									"\code{abs_sum_diff}" (default) or "\code{mahal_dist}."
+#' @param objective					The objective function to use when searching design space. This is a string
+#' 									with valid values "\code{mahal_dist}" (the default), "\code{abs_sum_diff}" or "\code{kernel}".
+#' @param Kgram						If the \code{objective = kernel}, this argument is required to be an \code{n x n} matrix whose
+#' 									entries are the evaluation of the kernel function between subject i and subject j. Default is \code{NULL}.
 #' @param wait						Should the \code{R} terminal hang until all \code{max_designs} vectors are found? The 
 #' 									default is \code{FALSE}.
 #' @param start						Should we start searching immediately (default is \code{TRUE}).
@@ -19,24 +24,32 @@
 #' 
 #' @author Adam Kapelner
 #' @export
-initRerandomizationExperimentalDesignObject = function(X, 
+initRerandomizationExperimentalDesignObject = function(
+		X = NULL, 
+		obj_val_cutoff_to_include,
 		max_designs = 1000,
-		obj_val_cutoff_to_include = NULL,
 		objective = "mahal_dist", 
+		Kgram = NULL,
 		wait = FALSE, 
 		start = TRUE,
 		num_cores = 1){
 	
-	#get dimensions immediately
-	n = nrow(X)
+	verify_objective_function(objective, Kgram, n)
+	
+	if (!is.null(Kgram)){
+		n = nrow(Kgram)
+		p = NA
+	} else {
+		n = nrow(X)
+		p = ncol(X)
+	}
 	if (n %% 2 != 0){
 		stop("Design matrix must have even rows to have equal treatments and controls")
 	}
-	p = ncol(X)
 	
 	if (objective == "abs_sum_diff"){
 		#standardize it -- much faster here
-		Xstd = apply(X, 2, function(xj){(xj - mean(xj)) / sd(xj)})
+		Xstd = standardize_data_matrix(X)
 	}
 	if (objective == "mahal_dist"){
 		if (p < n){
@@ -57,26 +70,35 @@ initRerandomizationExperimentalDesignObject = function(X,
 		.jcall(java_obj, "V", "setObjValCutoffToInclude", as.numeric(obj_val_cutoff_to_include))
 	}
 	.jcall(java_obj, "V", "setNumCores", as.integer(num_cores))
-	.jcall(java_obj, "V", "setNandP", as.integer(n), as.integer(p))
+	.jcall(java_obj, "V", "setN", as.integer(n))
+	if (objective != "kernel"){
+		p = ncol(X)
+		.jcall(java_obj, "V", "setP", as.integer(p))
+	}
 	.jcall(java_obj, "V", "setObjective", objective)
 	if (wait){
 		.jcall(java_obj, "V", "setWait")
-	}	
-	
-	#feed in the data
-	for (i in 1 : n){	
-		if (objective == "abs_sum_diff"){
-			.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), Xstd[i, , drop = FALSE]) #java indexes from 0...n-1
-		} else {
-			.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), X[i, , drop = FALSE]) #java indexes from 0...n-1
-		}
 	}
 	
-	#feed in the inverse var-cov matrix
-	if (objective == "mahal_dist"){
-		if (p < n){
-			for (j in 1 : p){
-				.jcall(java_obj, "V", "setInvVarCovRow", as.integer(j - 1), SinvX[j, , drop = FALSE]) #java indexes from 0...n-1
+	#feed in the gram matrix if applicable
+	if (!is.null(Kgram)){
+		setGramMatrix(java_obj, Kgram)
+	} else {
+		#feed in the raw data
+		for (i in 1 : n){
+			if (objective == "abs_sum_diff"){
+				.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), Xstd[i, , drop = FALSE]) #java indexes from 0...n-1
+			} else {
+				.jcall(java_obj, "V", "setDataRow", as.integer(i - 1), X[i, , drop = FALSE]) #java indexes from 0...n-1
+			}
+		}	
+		
+		#feed in the inverse var-cov matrix
+		if (objective == "mahal_dist"){
+			if (p < n){
+				for (j in 1 : p){
+					.jcall(java_obj, "V", "setInvVarCovRow", as.integer(j - 1), SinvX[j, , drop = FALSE]) #java indexes from 0...n-1
+				}
 			}
 		}
 	}
@@ -147,15 +169,19 @@ rerandomizationSearchCurrentProgress = function(obj){
 #' 
 #' @param obj 					The \code{rerandomization_experimental_design} object that is currently running the search
 #' @param include_assignments	Do we include the assignments (takes time) and default is \code{FALSE}.
+#' @param form					Which form should the assignments be in? The default is \code{one_zero} for 1/0's or \code{pos_one_min_one} for +1/-1's. 
 #' 
 #' @author Adam Kapelner
 #' @export
-resultsRerandomizationSearch = function(obj, include_assignments = FALSE){
+resultsRerandomizationSearch = function(obj, include_assignments = FALSE, form = "one_zero"){
 	obj_vals = .jcall(obj$java_obj, "[D", "getObjectiveVals")
 	
 	ending_indicTs = NULL
 	if (include_assignments){
-		ending_indicTs = sapply(.jcall(obj$java_obj, "[[I", "getEndingIndicTs"), .jevalArray)
+		ending_indicTs = .jcall(obj$java_obj, "[[I", "getEndingIndicTs", simplify = TRUE)
+		if (form == "pos_one_min_one"){
+			ending_indicTs = (ending_indicTs - 0.5) * 2
+		}
 	}	
 	
 	rerandomization_experimental_design_search_results = list(
